@@ -25,8 +25,11 @@
 #
 # HUBzero is a registered trademark of The Regents of the University of California.
 #
+__version__ = '3.10.1'
+
 import sys
 import os
+import json
 import logging
 import shutil
 import stat
@@ -48,22 +51,15 @@ from hubzero.submit.Probe              import Probe
 from hubzero.submit.ProbeQueue         import ProbeQueue
 
 class ProbeLauncher:
-   CONFIGURATIONDIRECTORY = os.path.join(os.sep,'etc','submit')
-   CONFIGURATIONFILE      = 'submit-probe.conf'
-   PROBELOGLOCATION       = "log"
-   PROBELOGFILE           = "probeLauncher.log"
-
    def __init__(self,
                 probeRoot,
-                probeLogLocation=PROBELOGLOCATION,
-                probeLogFile=PROBELOGFILE,
-                configurationDirectory=CONFIGURATIONDIRECTORY):
-      logDirectory = os.path.join(probeRoot,probeLogLocation)
-      self.openLogger(logDirectory,probeLogFile)
-      self.logger = logging.getLogger(__name__)
+                configurationDirectory,
+                configurationFilename):
+      self.logger = self.logger = logging.getLogger(__name__)
 
+      self.version             = __version__
       self.probeRoot           = probeRoot
-      self.configFilePath      = os.path.join(configurationDirectory,self.CONFIGURATIONFILE)
+      self.configFilePath      = os.path.join(configurationDirectory,configurationFilename)
       self.remoteProbeMonitors = []
       self.configData          = {}
 
@@ -89,6 +85,8 @@ remoteVenue=@@{REMOTEVENUE}
 probeId=@@{PROBEID}
 probeProgram=@@{PROBEPROGRAM}
 nCores=@@{NCORES}
+nGpus=@@{NGPUS}
+memory=@@{MEMORY}
 wallTime=@@{WALLTIME}
 inputs="@@{INPUTS}"
 preSubmitCommands="@@{PRESUBMITCOMMANDS}"
@@ -96,17 +94,28 @@ postSubmitCommands="@@{POSTSUBMITCOMMANDS}"
 
 coreArg=
 if [ ${nCores} -gt -1 ] ; then
-   coreArg="-nCpus ${nCores}"
+   coreArg="--nCpus ${nCores}"
 fi
 
-wallTimeArg="-wallTime 10"
+gpuArg=
+if [ ${nGpus} -gt -1 ] ; then
+   gpuArg="--nGpus ${nGpus}"
+fi
+
+memoryArg=
+if [ ${memory} -gt -1 ] ; then
+   memoryArg="--memory ${memory}"
+fi
+
+wallTimeArg="--wallTime 10"
 if [ -n "${wallTime}" ] ; then
    wallTimeArg="-w ${wallTime}"
 fi
+
 x509SubmitProxyArg=
-if [ -f ~/.globus/probe.proxy ] ; then
-   x509SubmitProxyArg="--x509SubmitProxy ~/.globus/probe.proxy"
-fi
+#if [ -f ~/.globus/probe.proxy ] ; then
+#   x509SubmitProxyArg="--x509SubmitProxy ~/.globus/probe.proxy"
+#fi
 
 #echo gridProbeRoot=${gridProbeRoot}
 #echo probeDirectory=${probeDirectory}
@@ -114,6 +123,8 @@ fi
 #echo probeId=${probeId}
 #echo probeProgram=${probeProgram}
 #echo nCores=${nCores}
+#echo nGpus=${nGpus}
+#echo memory=${memory}
 #echo wallTime=${wallTime}
 #echo inputs=${inputs}
 #echo preSubmitCommands=${preSubmitCommands}
@@ -142,20 +153,20 @@ printenv | sort
 pwd
 ls -lsa
 
-if [ "\${SITE_EXPECT_HOME_DIR}" = "" -o "\${SITE_EXPECT_HOME_DIR}" = "YES" ] ; then
-   if [ "\${HOME}" != "" ] ; then
-      cd \${HOME}
-      exitStatus=\$?
+if [ "\\${SITE_EXPECT_HOME_DIR}" = "" -o "\\${SITE_EXPECT_HOME_DIR}" = "YES" ] ; then
+   if [ "\\${HOME}" != "" ] ; then
+      cd \\${HOME}
+      exitStatus=\\$?
 
-      if [ \$exitStatus -eq 0 ] ; then
+      if [ \\$exitStatus -eq 0 ] ; then
          quota -s
-         exitStatus=\$?
-         if [ \$exitStatus -ne 0 ] ; then
+         exitStatus=\\$?
+         if [ \\$exitStatus -ne 0 ] ; then
             quota
-            exitStatus=\$?
-            if [ \$exitStatus -ne 0 ] ; then
+            exitStatus=\\$?
+            if [ \\$exitStatus -ne 0 ] ; then
                du -sh
-               exitStatus=\$?
+               exitStatus=\\$?
             fi
          fi
       else
@@ -164,7 +175,7 @@ if [ "\${SITE_EXPECT_HOME_DIR}" = "" -o "\${SITE_EXPECT_HOME_DIR}" = "YES" ] ; t
    fi
 fi
 
-exit \$exitStatus
+exit \\$exitStatus
 EOF
          chmod +x probegridsite.sh
       fi
@@ -190,6 +201,8 @@ EOF
 
       ${SUBMITPATH} --venue ${remoteVenue} \
                     ${coreArg} \
+                    ${gpuArg} \
+                    ${memoryArg} \
                     ${wallTimeArg} \
                     ${x509SubmitProxyArg} \
                     --disableProbeCheck \
@@ -301,6 +314,8 @@ exit ${exitStatus}
       self.maxJobFinished  = -1
       self.maxProbeStopped = -1
 
+      self.exitSystemCallOnInterrupt = False
+
       signal.signal(signal.SIGINT,self.sigINT_handler)
       signal.signal(signal.SIGHUP,self.sigHUP_handler)
       signal.signal(signal.SIGQUIT,self.sigQUIT_handler)
@@ -330,6 +345,10 @@ exit ${exitStatus}
          self.probeLaunchTerminated = True
          self.probeLaunchTerminating = False
 
+      if self.exitSystemCallOnInterrupt:
+         if sys.version_info > (3,):
+            raise InterruptedError(EINTR,"System Call Interrupt")
+
 
    def sigINT_handler(self,
                       signal,
@@ -363,47 +382,12 @@ exit ${exitStatus}
       self.sigGEN_handler(signal,frame)
 
 
-   def openLogger(self,
-                  logDirectory,
-                  probeLogFile):
-      class EmptyFilter(logging.Filter):
-         """
-         This is a filter which rejects empty messages
-
-         """
-
-         def filter(self,record):
-            if record.getMessage() == "":
-               emptyRecord = True
-            else:
-               emptyRecord = False
-
-            return(not emptyRecord)
-
-      APPLICATIONLOGGER = logging.getLogger('')
-      APPLICATIONLOGGER.setLevel(logging.DEBUG)
-
-      if not os.path.isdir(logDirectory):
-         os.makedirs(logDirectory)
-
-      probeLogPath = os.path.join(logDirectory,probeLogFile)
-      logHandler = logging.FileHandler(probeLogPath)
-      self.fdLogFile = logHandler.stream.fileno()
-
-      emptyFilter = EmptyFilter()
-      logHandler.addFilter(emptyFilter)
-
-      logFormatter = logging.Formatter('%(asctime)s %(message)s','[%a %b %d %H:%M:%S %Y]')
-      logHandler.setFormatter(logFormatter)
-      APPLICATIONLOGGER.addHandler(logHandler)
-
-
    def configure(self):
       configured = False
 
-      sectionPattern  = re.compile('(\s*\[)([^\s]*)(]\s*)')
-      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
-      commentPattern  = re.compile('\s*#.*')
+      sectionPattern  = re.compile(r'(\s*\[)([^\s]*)(]\s*)')
+      keyValuePattern = re.compile(r'( *)(\w*)( *= *)(.*[^\s$])( *)')
+      commentPattern  = re.compile(r'\s*#.*')
       inProbeSection  = False
 
       configured = False
@@ -428,7 +412,7 @@ exit ${exitStatus}
                                            'probeSitesFile':"probeSites.dat",
                                            'probeSiteShell':"./probesite.sh",
                                            'submitPath':"/usr/bin/submit",
-                                           'pegasusVersion':"4.5.2",
+                                           'pegasusVersion':"4.8.1",
                                            'remoteProbeMonitorHosts':[]
                                           }
                   elif inProbeSection:
@@ -448,20 +432,27 @@ exit ${exitStatus}
                                  sampleKey   = list(self.configData[key].keys())[0]
                                  sampleValue = self.configData[key][sampleKey]
                               except:
-                                 sampleKey   = "key"
-                                 sampleValue = "value"
-                              self.configData[key] = {} 
-                              for e in value.split(','):
-                                 dictKey,dictValue = e.split(':')
-                                 if isinstance(sampleKey,int):
-                                    dictKey = int(dictKey)
-                                 if   isinstance(sampleValue,int):
-                                    dictValue = int(dictValue)
-                                 elif isinstance(sampleValue,float):
-                                    dictValue = float(dictValue)
-                                 elif isinstance(sampleValue,bool):
-                                    dictValue = bool(dictValue.lower() == 'true')
-                                 self.configData[key][dictKey] = dictValue
+                                 try:
+                                    self.configData[key] = json.loads(value)
+                                 except:
+                                    self.configData[key] = {}
+                                    sampleKey   = "key"
+                                    sampleValue = "value"
+                              else:
+                                 self.configData[key] = {}
+               
+                              if not self.configData[key]:
+                                 for e in value.split(','):
+                                    dictKey,dictValue = e.split(':')
+                                    if isinstance(sampleKey,int):
+                                       dictKey = int(dictKey)
+                                    if   isinstance(sampleValue,int):
+                                       dictValue = int(dictValue)
+                                    elif isinstance(sampleValue,float):
+                                       dictValue = float(dictValue)
+                                    elif isinstance(sampleValue,bool):
+                                       dictValue = bool(dictValue.lower() == 'true')
+                                    self.configData[key][dictKey] = dictValue
                            else:
                               self.configData[key] = value
                         else:
@@ -492,39 +483,14 @@ exit ${exitStatus}
       return(configured)
 
 
-   def daemonize(self):
-      if self.fdLogFile != sys.stdout.fileno():
-         try:
-            devnull = open("/dev/null",'r')
-            try:
-               os.dup2(devnull.fileno(),sys.stdin.fileno())
-               os.dup2(self.fdLogFile,sys.stdout.fileno())
-               os.dup2(self.fdLogFile,sys.stderr.fileno())
-            except OSError:
-               self.logger.log(logging.ERROR,getLogMessage("file descriptor dup failed"))
-         except (IOError,OSError):
-            self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % ("/dev/null")))
-
-      if os.fork() != 0:
-         os.wait()
-         sys.exit(0)
-      else:
-         os.setsid()
-         pid = os.fork()
-         if pid != 0:
-            sys.exit(0)
-
-      time.sleep(2)
-
-
    def loadProbeSitesInfo(self):
       oldProbeSites = copy.copy(self.probeSites)
       del self.probeSites
       self.probeSites = {}
 
-      sitePattern     = re.compile('(.*\[)(.*)(])')
-      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
-      commentPattern  = re.compile('\s*#.*')
+      sitePattern     = re.compile(r'(.*\[)(.*)(])')
+      keyValuePattern = re.compile(r'( *)(\w*)( *= *)(.*[^\s$])( *)')
+      commentPattern  = re.compile(r'\s*#.*')
       siteName        = ""
 
       fpInfo = open(self.probeSitesPath,'r')
@@ -538,6 +504,8 @@ exit ${exitStatus}
                   siteName = sitePattern.match(record).group(2)
                   self.probeSites[siteName] = {'probeProgram':'./probegridsite.sh', \
                                                'nCores':-1, \
+                                               'nGpus':-1, \
+                                               'memory':-1, \
                                                'maximumTimeToProbeCompletion':-1, \
                                                'wallTime':10, \
                                                'inputs':[], \
@@ -621,6 +589,32 @@ exit ${exitStatus}
             fpDumpFile.close()
 
 
+   def __writeToStdout(self,
+                       message):
+      try:
+         try:
+            sys.stdout.write(message)
+         except UnicodeEncodeError:
+            sys.stdout.write(message.encode('utf-8'))
+         sys.stdout.flush()
+      except IOError as e:
+         if not e.args[0] in [EPIPE]:
+            self.logger.log(logging.ERROR,getLogMessage("Can't write to stdout: %s" % (message)))
+
+
+   def __writeToStderr(self,
+                       message):
+      try:
+         try:
+            sys.stderr.write(message)
+         except UnicodeEncodeError:
+            sys.stderr.write(message.encode('utf-8'))
+         sys.stderr.flush()
+      except IOError as e:
+         if not e.args[0] in [EPIPE]:
+            self.logger.log(logging.ERROR,getLogMessage("Can't write to stderr: %s" % (message)))
+
+
    def executeProbeCommand(self,
                            command,
                            environment=None,
@@ -673,22 +667,20 @@ exit ${exitStatus}
             os.kill(os.getpid(),signal.SIGTERM)
 
          if childoutFd in ready[0]:
-            outChunk = os.read(childoutFd,self.bufferSize)
+            outChunk = os.read(childoutFd,self.bufferSize).decode('utf-8')
             if outChunk == '':
                outEOF = True
             outData.append(outChunk)
             if streamOutput:
-               sys.stdout.write(outChunk)
-               sys.stdout.flush()
+               self.__writeToStdout(outChunk)
 
          if childerrFd in ready[0]:
-            errChunk = os.read(childerrFd,self.bufferSize)
+            errChunk = os.read(childerrFd,self.bufferSize).decode('utf-8')
             if errChunk == '':
                errEOF = True
             errData.append(errChunk)
             if streamOutput:
-               sys.stderr.write(errChunk)
-               sys.stderr.flush()
+               self.__writeToStderr(errChunk)
 
          if outEOF and errEOF:
             break
@@ -806,6 +798,7 @@ exit ${exitStatus}
                probe = self.toBeScheduledQueue.peek()
                maximumWaitTime = probe.when - time.time()
                if maximumWaitTime > 0.:
+                  self.exitSystemCallOnInterrupt = True
                   readyInputFds = select.select(inputDescriptors,[],[],maximumWaitTime)[0]
                else:
                   readyInputFds = []
@@ -814,10 +807,12 @@ exit ${exitStatus}
                readyInputFds = []
             else:
                raise
+         finally:
+            self.exitSystemCallOnInterrupt = False
 
          for readyInputFd in readyInputFds:
             probeId,probeSite,childPid,launchTime = self.remoteProbeJobInfo[readyInputFd]
-            message = os.read(readyInputFd,1)
+            message = os.read(readyInputFd,1).decode('utf-8')
 
             finishedPid,exitStatus = os.waitpid(childPid,0)
             if exitStatus != 0:
@@ -881,6 +876,8 @@ exit ${exitStatus}
                   substitutions["PROBEID"]            = "%09d" % (probe.id)
                   substitutions["PROBEPROGRAM"]       = self.probeSites[probe.destination]['probeProgram']
                   substitutions["NCORES"]             = self.probeSites[probe.destination]['nCores']
+                  substitutions["NGPUS"]              = self.probeSites[probe.destination]['nGpus']
+                  substitutions["MEMORY"]             = self.probeSites[probe.destination]['memory']
                   substitutions["WALLTIME"]           = self.probeSites[probe.destination]['wallTime']
                   substitutions["INPUTS"]             = inputs
                   substitutions["PRESUBMITCOMMANDS"]  = preSubmitCommands
@@ -893,6 +890,7 @@ exit ${exitStatus}
                   except KeyError as e:
                      probeSiteShellScript = ""
                      self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@{%s}\n" % (e.args[0])))
+                     self.logger.log(logging.ERROR,getLogMessage("in rawProbeSiteShellTemplate\n"))
                   except TypeError:
                      probeSiteShellScript = ""
                      self.logger.log(logging.ERROR,getLogMessage("Submission script substitution failed:\n%s\n" % \
@@ -936,7 +934,7 @@ exit ${exitStatus}
                      os.environ['PEGASUSVERSION'] = self.configData['pegasusVersion']
                      exitStatus,stdOutput,stdError = self.executeProbeCommand(probeCommandShellPath,
                                                                               environment=os.environ)
-                     os.write(childSendFd,"0")
+                     os.write(childSendFd,"0".encode('utf-8'))
                      try:
                         os.close(childSendFd)
                      except:
